/*
What's new:
C44 - [Martin Zoubek] first public release
*/

#ifndef _Types_h
  #define _Types_h

  typedef unsigned long pointer;

  #ifdef UNIX
    #include <stdlib.h>
  #else
    typedef unsigned int uint;
  #endif

  #ifdef WIN32
    #include <windows.h>
  #else
    typedef unsigned char BOOL;
    #define FALSE 0
    #define TRUE 1
  #endif

  typedef double double2[2];
  typedef double double3[3];
  typedef double double4[4];

  typedef double4 statevec[3];
  typedef double lintransf[3][2];
  typedef double bilintransf[3][2][3];
  typedef double4 viscousvector[3];     /*[3] = DOF */
  #undef min
  #undef max
  #define min(a,b) ((a)<(b)?(a):(b))
  #define max(a,b) ((a)>(b)?(a):(b))
  #define inabs(a) ((a)<(0)?-(a):(a))

  #define pow2(x) ((x)*(x))
  #define pow_fast(a,b) exp((b)*log(a))

  #define double2_dotprod(a,b) ((a)[0]*(b)[0]+(a)[1]*(b)[1])
  #define double2_norm2(a) double2_dotprod(a,a)
  #define double2_sub(c,a,b){\
    (c)[0]=(a)[0]-(b)[0];\
    (c)[1]=(a)[1]-(b)[1];\
    }
  #define double2_crossprod(c,a,b){\
    (c)=(a)[0]*(b)[1]-(a)[1]*(b)[0];\
    } 
  #define double2_distance2(a,b) (pow2((a)[0]-(b)[0])+pow2((a)[1]-(b)[1]))

  #define double2_gravicity(d,a,b,c){\
    (d)[0]=((a)[0]+(b)[0]+(c)[0])/3;\
    (d)[1]=((a)[1]+(b)[1]+(c)[1])/3;\
    }

  #define double2_weight_awer(d,a,b,p,q){\
    (d)[0]=(a)[0]*(p)+(b)[0]*(q);\
    (d)[1]=(a)[1]*(p)+(b)[1]*(q);\
    }

  #define double4_weight_awer(d,a,b,p,q){\
    (d)[0]=(a)[0]*(p)+(b)[0]*(q);\
    (d)[1]=(a)[1]*(p)+(b)[1]*(q);\
    (d)[2]=(a)[2]*(p)+(b)[2]*(q);\
    (d)[3]=(a)[3]*(p)+(b)[3]*(q);\
    }

  #define double4_add(c,a,b){\
    (c)[0]=(a)[0]+(b)[0];\
    (c)[1]=(a)[1]+(b)[1];\
    (c)[2]=(a)[2]+(b)[2];\
    (c)[3]=(a)[3]+(b)[3];\
    }
  #define double4_average(c,a,b){\
    (c)[0]=((a)[0]+(b)[0])/2;\
    (c)[1]=((a)[1]+(b)[1])/2;\
    (c)[2]=((a)[2]+(b)[2])/2;\
    (c)[3]=((a)[3]+(b)[3])/2;\
    }
  #define double4_sub(c,a,b){\
    (c)[0]=(a)[0]-(b)[0];\
    (c)[1]=(a)[1]-(b)[1];\
    (c)[2]=(a)[2]-(b)[2];\
    (c)[3]=(a)[3]-(b)[3];\
    }
  #define double4_multiply(c,r,a){\
    (c)[0]=(r)*(a)[0];\
    (c)[1]=(r)*(a)[1];\
    (c)[2]=(r)*(a)[2];\
    (c)[3]=(r)*(a)[3];\
    }
  #define double3_vertex(d,a,b,c){\
    (d)[0]=(a)[0]+(b)[0]-(c)[0];\
    (d)[1]=(a)[1]+(b)[1]-(c)[1];\
    (d)[2]=(a)[2]+(b)[2]-(c)[2];\
    }
  #define double3_vertex4(d,a,b,c){\
    (d)[0]=(a)[1]+(b)[1]-(c)[1];\
    (d)[1]=(a)[2]+(b)[2]-(c)[2];\
    (d)[2]=(a)[3]+(b)[3]-(c)[3];\
    }
  #define double4_vertex(d,a,b,c){\
    (d)[0]=(a)[0]+(b)[0]-(c)[0];\
    (d)[1]=(a)[1]+(b)[1]-(c)[1];\
    (d)[2]=(a)[2]+(b)[2]-(c)[2];\
    (d)[3]=(a)[3]+(b)[3]-(c)[3];\
    }

  #define vector_q2w(q,w,kappa){\
    (w)[0] = (q)[0];\
    (w)[1] = (q)[1]*(q)[0];\
    (w)[2] = (q)[2]*(q)[0];\
    (w)[3] = (q)[3]/((kappa)-1)+0.5*(q[0])*((q)[1]*(q)[1]+(q)[2]*(q)[2]);\
  }

#define compute_pressure(p,w,kappa){\
    (p)=(w)[3]-0.5*((w)[1]*(w)[1]+(w)[2]*(w)[2])/(w)[0])*((kappa)-1);\
  }

  #define double4_norm(c,a,b){\
    (c)=pow2((a)[0]-(b)[0])+pow2((a)[1]-(b)[1])+pow2((a)[2]-(b)[2])+pow2((a)[3]-(b)[3])  ;\
    }
  #define vector_1_w2q(w,q,kappa){\
     (q)[0] = (w)[0];\
     (q)[1] = (w)[1]/(w)[0];\
     (q)[2] = (w)[2]/(w)[0];\
     (q)[3] = sqrt(((w)[3]/(w)[0]-0.5*((q)[1]*(q)[1]+(q)[2]*(q)[2]))*kappa*(kappa-1));\
  }
 /*   (q)[3] = ((kappa)-1)*((w)[3]-0.5*((w)[1]*(w)[1]+(w)[2]*(w)[2])/(w)[0]);\ */


  #define vector_1_q2w(q,w,kappa){\
     (w)[0] = (q)[0];\
     (w)[1] = (q)[1]*(q)[0];\
     (w)[2] = (q)[2]*(q)[0];\
     (w)[3] = (q)[0]*((q)[3]*(q)[3]/kappa/((kappa)-1)+0.5*((q)[1]*(q)[1]+(q)[2]*(q)[2]));\
  }
/* (w)[3] = (q)[3]/((kappa)-1)+0.5*(q)[0]*((q)[1]*(q)[1]+(q)[2]*(q)[2]);\ */

#endif

