
void Solver_Flux_Roe(const tSolver_State *s,const double4 wi,
		     const double4 wj,const double2 n,double4 H){

  
  double qL[4],qR[4],f_roe[4],rlam[4],r[4][4],dw[5],gamma[4],f1[4],
    qqL,qqR,qqq,rhat,uhat,vhat,ehat,vhat2,ahat;

  /*   physical variables rhu,u,v,H (enthalpy)  */
  qL[0] = wi[0];
  qL[1] = ( n[0]*wi[1] + n[1]*wi[2])/wi[0];
  qL[2] = (-n[1]*wi[1] + n[0]*wi[2])/wi[0];
  qL[3] = (wi[3]*s->kappa - 0.5*(s->kappa-1)*
	   (wi[1]*wi[1]+wi[2]*wi[2])/wi[0])/wi[0];

  qR[0] = wj[0];
  qR[1] = ( n[0]*wj[1] + n[1]*wj[2])/wj[0];
  qR[2] = (-n[1]*wj[1] + n[0]*wj[2])/wj[0];
  qR[3] = (wj[3]*s->kappa - 0.5*(s->kappa-1)*
	   (wj[1]*wj[1]+wj[2]*wj[2])/wj[0])/wj[0];



 
 
  qqL = pow_fast(qL[0],0.5);
  qqR = pow_fast(qR[0],0.5);
  qqq = qqL+qqR;

  rhat = qqq/2;
  uhat = (qqL*qL[1] + qqR*qR[1] )/qqq;
  vhat = (qqL*qL[2] + qqR*qR[2] )/qqq;
  ehat = (qqL*qL[3] + qqR*qR[3] )/qqq;
  vhat2 = uhat*uhat+vhat*vhat;
  ahat = ((s->kappa-1)*(ehat-vhat2/2))**0.5;

      
  rlam[0] = uhat-ahat;
  rlam[1] = uhat;
  rlam[2] = uhat;
  rlam[3] = uhat+ahat;


  r[0][0] = 1.;
  r[0][1] = uhat-ahat;
  r[0][2] = vhat;
  r[0][3] = ehat-uhat*ahat;

  r[1][0] = 1.;
  r[1][1] = uhat;
  r[1][2] = vhat;
  r[1][3] = vhat2/2;

  r[2][0] = 0.;
  r[2][1] = 0.;
  r[2][2] = 1.;
  r[2][3] = vhat;

  r[3][0] = 1.;
  r[3][1] = uhat+ahat;
  r[3][2] = vhat;
  r[3][3] = ehat+uhat*ahat;

  for(k=0;k<4;k++)
    dw[k] = wj[k] - wi[k];
  
  dw[4] = dw[3] -(dw[2]-vhat*dw[0])*vhat;

  gamma[1] = (rkappa-1)/ahat/ahat*
    (dw[0]*(ehat-uhat*uhat) + uhat*dw[1]-dw[4]);
  gamma[0] = 0.5/ahat*(dw[0]*(uhat+ahat) - dw[1] - ahat*gamma[1]);

  gamma[3] = dw[0] - gamma[0] - gamma[1];
  gamma[2] = dw[2] - vhat*dw[0];


  pres = Pressure( wi,s->kappa-1);
  f[0] = wi[1];
  f(2) = wi[1]*wi[1]/wi[0]+pres;
  f(3) = wi[1]*wi[2]/wi[0]
  f(4) = (p + wi[3])*wi[1]/wi[0];

  for(k=0;k<4;k++){
    f_roe[k] = f1[k];
    for(j=0;j<4;j++){
      if(rlam[j] <= 0.)
	f_roe[k]+=  gamma[j]*rlam[j]*r[j][k];
    }
  }

  H[0] = f_roe[0];
  H[1] = f_roe[1]*n[0] - f_roe[2]*n[1];
  H[2] = f_roe[1]*n[1] + f_roe[2]*n[0];
  H[3] = f_roe[3];

}
